package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.distributed.print.business.bean.dto.CreatePdfDto;
import com.qmth.distributed.print.business.bean.dto.ExamStudentCourseDto;
import com.qmth.distributed.print.business.bean.dto.PaperPdfDto;
import com.qmth.distributed.print.business.bean.dto.PdfDto;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.PdfTaskLogicService;
import com.qmth.distributed.print.business.util.pdf.CreatePdfNewUtil;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.entity.BasicPrintConfig;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.enums.CreatePdfTypeEnum;
import com.qmth.teachcloud.common.enums.ExamCategoryEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FlowStatusEnum;
import com.qmth.teachcloud.common.service.BasicAttachmentService;
import com.qmth.teachcloud.common.service.BasicStudentService;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.util.ExamTaskUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class PdfTaskLogicServiceImpl implements PdfTaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(PdfTaskLogicServiceImpl.class);

    @Resource
    ExamPrintPlanService examPrintPlanService;
    @Resource
    ExamTaskService examTaskService;
    @Resource
    ExamTaskDetailService examTaskDetailService;
    @Resource
    ExamCardService examCardService;
    @Resource
    ExamStudentService examStudentService;
    @Resource
    BasicAttachmentService basicAttachmentService;
    @Resource
    ExamDetailService examDetailService;
    @Resource
    CommonCacheService commonCacheService;
    @Resource
    CreatePdfNewUtil createPdfNewUtil;
    @Resource
    BasicCardRuleService basicCardRuleService;
    @Autowired
    ExamDetailCourseService examDetailCourseService;
    @Autowired
    BasicStudentService basicStudentService;
    @Autowired
    ExamTaskPrintService examTaskPrintService;
    @Resource
    BasicExamService basicExamService;
    @Resource
    ExamTaskAssignPaperTypeService examTaskAssignPaperTypeService;

    /**
     * 创建A4文件
     *
     * @param tbTaskPdf
     * @param examDetailCourseList
     * @throws Exception
     */
    @Transactional
    public void createA4File(TBTaskPdf tbTaskPdf, List<ExamDetailCourse> examDetailCourseList) throws Exception {
        CreatePdfTypeEnum createPdfType = tbTaskPdf.getCreateType();
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTaskPdf.getPrintPlanId());
        ExamDetail examDetail = examDetailService.getById(tbTaskPdf.getId());
        BasicSchool basicSchool = commonCacheService.schoolCache(tbTaskPdf.getSchoolId());
        // 印品
        String ordinaryContent = examPrintPlan.getOrdinaryContent();
        if ((CreatePdfTypeEnum.ALL.equals(createPdfType) || CreatePdfTypeEnum.CHECK_IN.equals(createPdfType)) && StringUtils.isNotBlank(ordinaryContent)) {
            //获取普通印品
            JSONArray jsonArrayOrdinary = JSONArray.parseArray(ordinaryContent);
            for (int i = 0; i < jsonArrayOrdinary.size(); i++) {
                JSONObject jsonObjectOrdinary = jsonArrayOrdinary.getJSONObject(i);
                if (Objects.nonNull(jsonObjectOrdinary.get("attachmentId")) && !Objects.equals("", jsonObjectOrdinary.get("attachmentId"))) {
                    Long attachmentId = Long.parseLong((String) jsonObjectOrdinary.get("attachmentId"));
                    BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                    createPdfNewUtil.createCheckIn(examDetail, basicAttachment, (Integer) jsonObjectOrdinary.get("backupCount"));
                }
            }
        }
        BasicExam basicExam = basicExamService.getById(examPrintPlan.getExamId());
        if (basicExam.getCategory().equals(ExamCategoryEnum.FORMAL)) {
            String variableContent = examPrintPlan.getVariableContent();
            if (StringUtils.isNotBlank(variableContent)) {
                List<Long> examDetailCourseListIds = examDetailCourseList.stream().map(BaseEntity::getId).collect(Collectors.toList());
                List<ExamStudentCourseDto> examStudentCourseDtoList = examStudentService.queryBySchoolIdAndExamDetailCourseIds(basicSchool.getId(), examDetailCourseListIds);
                //获取变量印品
                JSONArray jsonArrayVariable = JSONArray.parseArray(variableContent);
                for (int i = 0; i < jsonArrayVariable.size(); i++) {
                    JSONObject jsonObjectVariable = jsonArrayVariable.getJSONObject(i);
                    String type = (String) jsonObjectVariable.get("type");
                    if (Objects.nonNull(jsonObjectVariable.get("templateId")) && !Objects.equals("", jsonObjectVariable.get("templateId"))) {
                        Long templateId = Long.parseLong((String) jsonObjectVariable.get("templateId"));
                        if ((CreatePdfTypeEnum.ALL.equals(createPdfType) || CreatePdfTypeEnum.SIGN.equals(createPdfType)) && Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "SIGN")) {//签到表
                            createPdfNewUtil.createSignBook(templateId, basicSchool.getName(), examDetail, examStudentCourseDtoList, (Integer) jsonObjectVariable.get("backupCount"), examDetailCourseList);
                        } else if ((CreatePdfTypeEnum.ALL.equals(createPdfType) || CreatePdfTypeEnum.PACKAGE.equals(createPdfType)) && Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "PACKAGE")) {//卷袋贴
                            Integer backupCount = SystemConstant.calcBackupCount(examDetail.getBackupCount(), examDetail.getTotalSubjects(), 1);
                            createPdfNewUtil.createPaperPackage(templateId, basicSchool.getName(), examDetail, examStudentCourseDtoList, backupCount, (Integer) jsonObjectVariable.get("backupCount"), examDetailCourseList, examPrintPlan.getExamId());
                        }
                    }
                }
            }
        }
        examDetailService.updateById(examDetail);
    }

    @Override
    public void executeCreatePdfLogic(TBTaskPdf tbTaskPdf) {
        try {
            // 更新状态（考场状态、印刷计划状态）
            updatePdfDataStatus(tbTaskPdf);
            List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.listByExamDetailId(tbTaskPdf.getId());
            // 抽取卷型，更新考场、考生对应卷型值
            updateAssignPaperType(tbTaskPdf, examDetailCourseList);
            // 查询生成pdf需要文件
            CreatePdfDto createPdfDto = new CreatePdfDto();
            assemblePdfFile(tbTaskPdf, createPdfDto);
            // 生成pdf文件并合并
            //合并（试卷+备用试卷）
            String dirNamePaper = createPdfNewUtil.mergeA3Pdf(createPdfDto.getPaperPdfList(), createPdfDto.getBackupPaperPdfList());
            //合并A3（题卡+备用题卡）
            String dirNameCardA3 = createPdfNewUtil.mergeA3Pdf(createPdfDto.getCardPdfList(), createPdfDto.getBackupCardPdfList());
            // 生成A4文件
            createA4File(tbTaskPdf, examDetailCourseList);
            // 更新文件路径、更新考场状态、印刷计划状态
            createPdfNewUtil.mergePdfSaveDb(dirNamePaper, dirNameCardA3, tbTaskPdf);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private void assemblePdfFile(TBTaskPdf tbTaskPdf, CreatePdfDto createPdfDto) throws Exception {
        List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.listByExamDetailId(tbTaskPdf.getId());
        for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
            String[] paperTypes = examDetailCourse.getPaperType().split(",");

            List<PdfDto> studentPaperPdfList = new ArrayList<>();//所有试卷
            List<PdfDto> studentCardPdfList = new ArrayList<>();//所有题卡
            List<PdfDto> backupPaperPdfList = new ArrayList<>();//备份试卷
            List<PdfDto> backupCardPdfList = new ArrayList<>();//备份题卡

            //查询命题任务绑定的试卷和题卡
            ExamTask examTask = examTaskService.getByCourseCodeAndPaperNumber(tbTaskPdf.getSchoolId(), tbTaskPdf.getExamId(), examDetailCourse.getCourseCode(), examDetailCourse.getPaperNumber());
            if (examTask == null) {
                throw ExceptionResultEnum.EXAM_TASK_IS_NULL.exception();
            } else if (examTask.getReview()) {
                examTask = examTaskService.findExamTaskByFlowStatus(tbTaskPdf.getSchoolId(), tbTaskPdf.getExamId(), examDetailCourse.getCourseCode(), examDetailCourse.getPaperNumber(), FlowStatusEnum.FINISH);
            } else {
                if (!ExamStatusEnum.SUBMIT.equals(examTask.getStatus())) {
                    throw ExceptionResultEnum.EXAM_TASK_NOT_SUBMIT.exception();
                } else if (!examTask.getEnable()) {
                    throw ExceptionResultEnum.EXAM_TASK_ENABLE.exception();
                }
            }
            ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamTaskId(examTask.getId());

            Map<String, ExamCard> examCardMap = new HashMap<>();
            List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds());
            ExamCard examCard;
            for (PaperInfoVo paperInfoVo : paperInfoVoList) {
                Long cardId = Long.valueOf(paperInfoVo.getCardId());
                if (Objects.nonNull(cardId)) {
                    examCard = examCardService.getById(cardId);
                    if (Objects.isNull(examCard)) {
                        throw ExceptionResultEnum.EXAM_CARD_IS_NULL.exception();
                    }
                    examCardMap.put(paperInfoVo.getName(), examCard);
                }
            }

            //查询题卡规则
            BasicCardRule basicCardRule = basicCardRuleService.getById(examTask.getCardRuleId());

            List<ExamStudent> examStudentList = examStudentService.listByExamDetailCourseId(examDetailCourse.getId());

            CreatePdfTypeEnum createPdfType = tbTaskPdf.getCreateType();

            ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTaskPdf.getPrintPlanId());
            String printContent = examPrintPlan.getPrintContent();
            if (StringUtils.isBlank(printContent)) {
                continue;
            }

            // 计算备份数量，默认最小为1份。
            ExamDetail examDetail = examDetailService.getById(tbTaskPdf.getId());
            int backupCount = SystemConstant.calcBackupCount(examDetail.getBackupCount(), examDetail.getTotalSubjects(), 1);

            // 试卷数据组装
            if ((CreatePdfTypeEnum.ALL.equals(createPdfType) || CreatePdfTypeEnum.PAPER.equals(createPdfType)) && printContent.contains("PAPER")) {
                List<PaperPdfDto> paperPdfDto = createPdfNewUtil.getPaperPdfFile(examDetailCourse.getPaperType(), examTaskDetail, createPdfDto.getFileTempList());

                //获取试卷pdf
                PdfDto pdfDto = createPdfNewUtil.getPaperPdf(paperPdfDto, backupCount, backupPaperPdfList);
                if (Objects.nonNull(pdfDto)) {
                    examDetailCourse.setPaperPagesA3(pdfDto.getPageCount());
                } else {
                    examDetailCourse.setPaperPagesA3(examDetailCourse.getPaperPagesA3());
                }
                // 备用试卷
                createPdfDto.getBackupPaperPdfList().addAll(backupPaperPdfList);

                if (examStudentList != null && examStudentList.size() > 0) {
                    for (ExamStudent t : examStudentList) {
                        if (Objects.nonNull(pdfDto)) {
                            String[] waterMarkNames = {t.getStudentName(), t.getTicketNumber()};
                            createPdfNewUtil.getExamStudentPaperPdf(t.getPaperType(), paperPdfDto, studentPaperPdfList, waterMarkNames);
                        }
                    }
                } else if (examDetail.getTotalSubjects() != null) {
                    AtomicInteger atomicInteger = new AtomicInteger(0);
                    int i = 0;
                    while (i < examDetail.getTotalSubjects()) {
                        int seq = atomicInteger.getAndIncrement();
                        int mod = seq % paperTypes.length;
                        if (Objects.nonNull(pdfDto)) {
                            createPdfNewUtil.getExamStudentPaperPdf(paperTypes[mod], paperPdfDto, studentPaperPdfList, null);
                        }
                        i++;
                    }
                } else {
                    throw ExceptionResultEnum.ERROR.exception("数据错误：未找到考生或者印刷数量");
                }
                // 考生试卷
                createPdfDto.getPaperPdfList().addAll(studentPaperPdfList);
            }

            // 题卡数据组装
            if ((CreatePdfTypeEnum.ALL.equals(createPdfType) || CreatePdfTypeEnum.CARD_A3.equals(createPdfType)) && printContent.contains("CARD")) {
                Map<String, ExamCard> examCardDetailMap = new HashMap<>();
                Map<String, String> cardContentMap = new HashMap<>();
                JSONObject jsonObject = new JSONObject();
                JSONArray jsonArray = new JSONArray();
                JSONArray stuJsonArray = new JSONArray();
                for (String s : paperTypes) {
                    examCard = examCardMap.get(s);
                    Optional.ofNullable(examCard).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("卷型" + s + "题卡不存在"));

                    //把模板页面上的 ${} 替换成空
                    String cardContent = createPdfNewUtil.resetHtmlTemplateBar(examCard.getHtmlContent());

                    for (int i = 1; i <= backupCount; i++) {
                        String packageCode = examDetail.getPackageCode() + String.format(SystemConstant.DATE_TIME_FORMAT, i);
                        BasicAttachment basicAttachment = createPdfNewUtil.cardHtml(packageCode, cardContent, s, tbTaskPdf.getCreateId(), backupCardPdfList, basicCardRule);
                        examDetailCourse.setCardPagesA3(basicAttachment.getPages());

                        JSONObject object = new JSONObject();
                        object.put("name", s);
                        object.put("attachmentId", basicAttachment.getId());
                        object.put("packageCode", packageCode);
                        jsonArray.add(object);
                    }
                    examCardDetailMap.put(s, examCard);
                    cardContentMap.put(s, cardContent);
                }
                jsonObject.put("card", jsonArray);
                examDetailCourse.setAttachmentId(jsonObject.toJSONString());

                // 备用题卡
                createPdfDto.getBackupCardPdfList().addAll(backupCardPdfList);

                if (examStudentList != null && examStudentList.size() > 0) {
                    for (ExamStudent t : examStudentList) {
                        // 用带条码的模板
                        createPdfNewUtil.examStudentHtml(examCardDetailMap.get(t.getPaperType()).getHtmlContent(), t, t.getPaperType(), examDetail, tbTaskPdf.getCreateId(), studentCardPdfList, basicCardRule);
                    }
                    examStudentService.updateBatchById(examStudentList);
                } else if (examDetailCourse.getTotalSubjects() != null) {
                    AtomicInteger atomicInteger = new AtomicInteger(0);
                    Map<String, BasicAttachment> stringBasicAttachmentMap = new HashMap<>();
                    int i = 0;
                    while (i < examDetailCourse.getTotalSubjects()) {
                        int seq = atomicInteger.getAndIncrement();
                        int mod = seq % examTaskDetail.getDrawCount();
                        String tempPaperType = paperTypes[mod];
                        BasicAttachment basicAttachment = createPdfNewUtil.examStudentHtml(cardContentMap.get(tempPaperType), null, tempPaperType, examDetail, tbTaskPdf.getCreateId(), studentCardPdfList, basicCardRule);

                        if (!stringBasicAttachmentMap.containsKey(tempPaperType)) {
                            stringBasicAttachmentMap.put(tempPaperType, basicAttachment);
                            JSONObject object = new JSONObject();
                            object.put("name", tempPaperType);
                            object.put("attachmentId", basicAttachment.getId());
                            stuJsonArray.add(object);

                            // 用不带条码的模板
//                            basicAttachmentList.add(basicAttachment);
                        }
                        i++;
                    }
                    examDetailCourse.setCommonAttachmentId(stuJsonArray.toJSONString());
                } else {
                    throw ExceptionResultEnum.ERROR.exception("数据错误：未找到考生或者印刷数量");
                }
                // 题卡
                createPdfDto.getCardPdfList().addAll(studentCardPdfList);
            }
        }
    }

    @Transactional
    public void updateAssignPaperType(TBTaskPdf tbTaskPdf, List<ExamDetailCourse> examDetailCourseList) {

        for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
            ExamTaskAssignPaperType examTaskAssignPaperType = examTaskAssignPaperTypeService.extractPaperType(tbTaskPdf, examDetailCourse);
            List<ExamStudent> examStudentList = examStudentService.listByExamDetailCourseId(examDetailCourse.getId());
            // 考生实际关联试卷类型
            List<String> relatePaperTypes = new ArrayList<>();
            List<String> paperTypes = Arrays.stream(examTaskAssignPaperType.getPaperType().split(",")).sorted(Comparator.comparing(String::valueOf)).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(examStudentList)) {
                AtomicInteger atomicInteger = new AtomicInteger(0);
                for (ExamStudent examStudent : examStudentList) {
                    int i1 = atomicInteger.getAndIncrement();
                    int mod = i1 % paperTypes.size();
                    examStudent.setPaperType(paperTypes.get(mod));
                    if (!relatePaperTypes.contains(examStudent.getPaperType())) {
                        relatePaperTypes.add(examStudent.getPaperType());
                    }
                }
            } else if (examDetailCourse.getTotalSubjects() != null) {
                if (examDetailCourse.getTotalSubjects().intValue() - paperTypes.size() >= 0) {
                    relatePaperTypes.addAll(paperTypes);
                } else {
                    for (int i = 0; i < examDetailCourse.getTotalSubjects().intValue(); i++) {
                        relatePaperTypes.add(paperTypes.get(i));
                    }
                }
            } else {
                relatePaperTypes.addAll(paperTypes);
            }

            // 更新卷型数据
            if (!CollectionUtils.isEqualCollection(paperTypes, relatePaperTypes)) {
                examTaskAssignPaperType.setPaperType(String.join(",", relatePaperTypes));
                examTaskAssignPaperTypeService.updateByMultiId(examTaskAssignPaperType);
            }

            examDetailCourse.setPaperType(String.join(",", relatePaperTypes));
            examDetailCourseService.updateById(examDetailCourse);

            examStudentService.updateBatchById(examStudentList);
        }
    }

    private void updatePdfDataStatus(TBTaskPdf tbTaskPdf) {
        //查询printPlan
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTaskPdf.getPrintPlanId());
        if (Objects.isNull(examPrintPlan)) {
            throw ExceptionResultEnum.EXAM_PRINT_IS_NULL.exception();
        }
        UpdateWrapper<ExamDetail> examDetailQueryWrapper = new UpdateWrapper<>();
        examDetailQueryWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.CREATING)
                .eq(ExamDetail::getId, tbTaskPdf.getId());
        examDetailService.update(examDetailQueryWrapper);

        //所有考场都撤回，印刷任务状态改为就绪
        QueryWrapper<ExamDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetail::getPrintPlanId, tbTaskPdf.getPrintPlanId())
                .notIn(ExamDetail::getStatus, ExamDetailStatusEnum.NEW, ExamDetailStatusEnum.READY);
        List<ExamDetail> examDetails = examDetailService.list(queryWrapper);
        if (CollectionUtils.isEmpty(examDetails)) {
            examPrintPlanService.updateStatusById(tbTaskPdf.getPrintPlanId(), PrintPlanStatusEnum.READY);
        }
    }

    /**
     * 更新考试计划
     *
     * @param basicPrintConfig
     * @param examPrintPlan
     */
    @Transactional
    public void updateExamPrintPlan(BasicPrintConfig basicPrintConfig, ExamPrintPlan examPrintPlan) {

    }

}
